import { Clipboard } from "clipeasy";
var GeneralDsu = /** @class */ (function () {
    function GeneralDsu() {
        /**
         * 集合存储域
         */
        this._set = [];
        /**
         * 判等依据
         * @param data
         * @param target
         * @returns
         */
        this._equalCondition = function (data, target) {
            return data === target;
        };
        var template = "\ninterface GeneralDsuElement<T> {\n  /**\n   * \u6570\u636E\u57DF\n   */\n  dat: T;\n  /**\n   * \u7236\u8282\u70B9\u7684\u7D22\u5F15\uFF0C\u5982\u679C\u4E3A\u8D1F\u6570\uFF0C\u5219\u6CA1\u6709\u7236\u8282\u70B9\n   */\n  parent: number;\n}\n\nclass GeneralDsu<T> {\n  /**\n   * \u96C6\u5408\u5B58\u50A8\u57DF\n   */\n  private _set: GeneralDsuElement<T>[] = [];\n\n  /**\n   * \u5224\u7B49\u4F9D\u636E\n   * @param data\n   * @param target\n   * @returns\n   */\n  private _equalCondition = (data: T, target: T) => {\n    return data === target;\n  };\n\n  /**\n   * \u8BBE\u7F6E\u5224\u7B49\u6761\u4EF6\n   * @param outerEqualCondition\n   */\n  public setEqual(outerEqualCondition: (data: T, target: T) => boolean) {\n    this._equalCondition = outerEqualCondition;\n  }\n\n  /**\n   * \u521D\u59CB\u5316\u5E76\u67E5\u96C6\n   * @param values\n   */\n  init(values: T[]) {\n    values.forEach((v) => {\n      this._set.push({\n        // \u521D\u59CB\u5316\u7684\u65F6\u5019\uFF0C\u6BCF\u4E2A\u5B50\u6811\u53EA\u6709\u4E00\u4E2A\u5143\u7D20\n        parent: -1,\n        dat: v,\n      });\n    });\n  }\n\n  /**\n   * \u67E5\u627E\u5143\u7D20\u662F\u5426\u5728\u96C6\u5408\u4E2D\uFF0C\u82E5\u5B58\u5728\uFF0C\u5219\u8FD4\u56DE\u6839\u8282\u70B9\u6240\u5728\u7684\u7D22\u5F15\uFF0C\u82E5\u4E0D\u5728\uFF0C\u5219\u8FD4\u56DE-1\n   * @param target \u76EE\u6807\u5143\u7D20\n   */\n  find(target: T): number {\n    for (let i = 0; i < this._set.length; i++) {\n      // \u5143\u7D20\u5728\u6570\u7EC4\u4E2D\u80FD\u591F\u88AB\u627E\u5230\n      if (this._equalCondition(this._set[i].dat, target)) {\n        let pos = i;\n        // \u5C1D\u8BD5\u627E\u8FD9\u4E2A\u5143\u7D20\u7684\u7956\u5148\u8282\u70B9\n        while (this._set[pos].parent >= 0) {\n          // \u7EE7\u7EED\u5411\u4E0A\u67E5\u627E\u6839\u8282\u70B9\n          pos = this._set[pos].parent;\n        }\n        // \u8DEF\u5F84\u538B\u7F29\uFF1A\u5C06\u8DEF\u5F84\u4E0A\u7684\u8282\u70B9\u90FD\u76F4\u63A5\u8FDE\u63A5\u5230\u6839\u8282\u70B9\n        let current = i;\n        while (current !== pos) {\n          const next = this._set[current].parent;\n          this._set[current].parent = pos;\n          current = next;\n        }\n        return pos;\n      }\n    }\n    // \u672A\u627E\u5230\n    return -1;\n  }\n\n  /**\n   * \u5408\u5E76\u96C6\u5408\n   * @param set1\n   * @param set2\n   */\n  union(set1: T, set2: T) {\n    const r1 = this.find(set1);\n    const r2 = this.find(set2);\n    if (r1 != r2) {\n      const val1 = Math.abs(this._set[r1].parent);\n      const val2 = Math.abs(this._set[r2].parent);\n      // \u5C06\u5C0F\u6811\u8D34\u5230\u5927\u6811\u4E0A\n      if (val1 < val2) {\n        this._set[r1].parent = r2;\n      } else {\n        // \u5982\u679C\u4E24\u68F5\u6811\u76F8\u540C\uFF0C\u5219\u9700\u8981\u5C06\u6811\u7684\u89C4\u6A21\u589E\u52A0\n        if (val1 === val2) {\n          this._set[r1].parent--;\n        }\n        this._set[r2].parent = r1;\n      }\n    }\n  }\n\n  /**\n   * \u7EDF\u8BA1\u5E76\u67E5\u96C6\u4E2D\u5B50\u6811\u7684\u4E2A\u6570\n   * @returns\n   */\n  counter() {\n    let size = 0;\n    this._set.forEach((v) => {\n      if (v.parent < 0) {\n        size++;\n      }\n    });\n    return size;\n  }\n}\n";
        var clip = new Clipboard();
        clip.write(template);
    }
    /**
     * 设置判等条件
     * @param outerEqualCondition
     */
    GeneralDsu.prototype.setEqual = function (outerEqualCondition) {
        this._equalCondition = outerEqualCondition;
    };
    /**
     * 初始化并查集
     * @param values
     */
    GeneralDsu.prototype.init = function (values) {
        var _this = this;
        values.forEach(function (v) {
            _this._set.push({
                // 初始化的时候，每个子树只有一个元素
                parent: -1,
                dat: v,
            });
        });
    };
    /**
     * 查找元素是否在集合中，若存在，则返回根节点所在的索引，若不在，则返回-1
     * @param target 目标元素
     */
    GeneralDsu.prototype.find = function (target) {
        for (var i = 0; i < this._set.length; i++) {
            // 元素在数组中能够被找到
            if (this._equalCondition(this._set[i].dat, target)) {
                var pos = i;
                // 尝试找这个元素的祖先节点
                while (this._set[pos].parent >= 0) {
                    // 继续向上查找根节点
                    pos = this._set[pos].parent;
                }
                // 路径压缩：将路径上的节点都直接连接到根节点
                var current = i;
                while (current !== pos) {
                    var next = this._set[current].parent;
                    this._set[current].parent = pos;
                    current = next;
                }
                return pos;
            }
        }
        // 未找到
        return -1;
    };
    /**
     * 合并集合
     * @param set1
     * @param set2
     */
    GeneralDsu.prototype.union = function (set1, set2) {
        var r1 = this.find(set1);
        var r2 = this.find(set2);
        if (r1 != r2) {
            var val1 = Math.abs(this._set[r1].parent);
            var val2 = Math.abs(this._set[r2].parent);
            // 将小树贴到大树上
            if (val1 < val2) {
                this._set[r1].parent = r2;
            }
            else {
                // 如果两棵树相同，则需要将树的规模增加
                if (val1 === val2) {
                    this._set[r1].parent--;
                }
                this._set[r2].parent = r1;
            }
        }
    };
    /**
     * 统计并查集中子树的个数
     * @returns
     */
    GeneralDsu.prototype.counter = function () {
        var size = 0;
        this._set.forEach(function (v) {
            if (v.parent < 0) {
                size++;
            }
        });
        return size;
    };
    return GeneralDsu;
}());
export { GeneralDsu };
