var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
/*
 * 抽象堆
 */
var Heap = /** @class */ (function () {
    function Heap() {
        var _a;
        var _this = this;
        var initElements = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            initElements[_i] = arguments[_i];
        }
        /**
         * 定义一个存储数据的内存空间
         */
        this._data = [];
        this._selfCompare = function (compareVal, currentVal) {
            if (typeof _this._compare !== "function") {
                throw "cannot compare without a compare function";
            }
            return _this._compare(compareVal, currentVal);
        };
        (_a = this._data).push.apply(_a, __spreadArray([], __read(initElements), false));
    }
    Object.defineProperty(Heap.prototype, "count", {
        get: function () {
            return this._data.length;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * 设置比较函数
     * @param {(compareVal, currentVal) => boolean} compareFunc
     */
    Heap.prototype.setCompare = function (compareFunc) {
        this._compare = compareFunc;
    };
    /**
     * 获取堆顶的元素
     */
    Heap.prototype.getTop = function () {
        return this._data[0];
    };
    /**
     * 按断堆是否是空
     */
    Heap.prototype.isEmpty = function () {
        return this._data.length === 0;
    };
    /**
     * 向堆中插入一个合法值
     * @param  val
     */
    Heap.prototype.insertQueue = function (val) {
        // 如果当前数组没有元素，直接插入即可
        if (this._data.length === 0) {
            this._data[0] = val;
        }
        else {
            // 让i指向当前新位置，因为没有哨兵的关系，最后一个元素是length - 1，所以新位置就是length
            var i = this.count;
            while (this._selfCompare(val, this._data[Math.floor(i / 2)]) && i > 0) {
                this._data[i] = this._data[Math.floor(i / 2)];
                i = Math.floor(i / 2);
            }
            // 在合适的位置插入新的值
            this._data[i] = val;
        }
    };
    /**
     * 获取堆中的最值元素
     * @returns
     */
    Heap.prototype.deleteTop = function () {
        if (this.isEmpty()) {
            throw "can not delete element from empty heap";
        }
        // 取出堆顶的元素
        var minVal = this._data[0];
        var temp = this._data[this.count - 1];
        this._data[0] = temp;
        // JavaScript语言需要进行这一步，让数组的规模缩小，释放空间
        this._data.length--;
        // 如果当前堆里面还存在元素的话，将
        this.percDown(0);
        return minVal;
    };
    /**
     * 下滤：将堆中以堆data[p]为根的子堆调整为最小堆
     * @param  p 根节点索引
     */
    Heap.prototype.percDown = function (p) {
        // 如果当前堆元素小于1个，就不执行调整操作
        if (this.count <= 1) {
            return;
        }
        var parent, child;
        /* 取出根结点存放的值 */
        var temp = this._data[p];
        for (parent = p; parent * 2 < this.count; parent = child) {
            child = parent * 2;
            if (child + 1 < this.count && this._selfCompare(this._data[child + 1], this._data[child])) {
                child++; /* child指向左右子结点的较大者 最大堆 较小者 最小堆 */
            }
            /* 找到了合适位置 */
            // 注意这儿一定要取得等号 temp <= this.#data[child] 最小堆 temp >= this.#data[child] 最大堆
            if (this._selfCompare(temp, this._data[child])) {
                break;
            }
            else {
                /* 下滤X */
                this._data[parent] = this._data[child];
            }
        }
        this._data[parent] = temp;
    };
    /**
     * 构建最小堆
     */
    Heap.prototype.buildHeap = function () {
        /* 调整data中的元素，使满足最小堆的有序性  */
        /* 这里所有size个元素已经存在data[]中 */
        /* 从最后一个结点的父节点开始，到根结点1 */
        for (var i = Math.floor((this.count - 1) / 2); i >= 0; i--) {
            this.percDown(i);
        }
    };
    return Heap;
}());
export { Heap };
